@@ -35,50 +35,56 @@
 #include "utils/memutils.h"
 #include "context/pool_session_context.h"
 
-typedef struct {
-	char	*attrname;	/* attribute name */
-	char	*adsrc;		/* default value expression */
-	int		 use_timestamp;	/* not zero if timestamp is used in default value */
-} TSAttr;
+typedef struct
+{
+	char	   *attrname;		/* attribute name */
+	char	   *adsrc;			/* default value expression */
+	int			use_timestamp;	/* not zero if timestamp is used in default
+								 * value */
+}			TSAttr;
 
-typedef struct {
-	int		relnatts;	/* num of attributes */
-	TSAttr	attr[1];
-} TSRel;
+typedef struct
+{
+	int			relnatts;		/* num of attributes */
+	TSAttr		attr[1];
+}			TSRel;
 
-typedef struct {
-	A_Const					*ts_const;
-	POOL_CONNECTION_POOL	*backend;
-	char					*relname;
-	int						 num_params;		/* num of original params (for Parse) */
-	bool		 			 rewrite_to_params;	/* true if timestamp is rewritten to param insread of const */
-	bool		 			 rewrite;			/* has rewritten? */
-	List					*params;			/* list of additional params */
-} TSRewriteContext;
+typedef struct
+{
+	A_Const    *ts_const;
+	POOL_CONNECTION_POOL *backend;
+	char	   *relname;
+	int			num_params;		/* num of original params (for Parse) */
+	bool		rewrite_to_params;	/* true if timestamp is rewritten to param
+									 * insread of const */
+	bool		rewrite;		/* has rewritten? */
+	List	   *params;			/* list of additional params */
+}			TSRewriteContext;
 
-static void *ts_register_func(POOL_SELECT_RESULT *res);
+static void *ts_register_func(POOL_SELECT_RESULT * res);
 static void *ts_unregister_func(void *data);
-static TSRel *relcache_lookup(TSRewriteContext *ctx);
+static TSRel * relcache_lookup(TSRewriteContext * ctx);
 static bool isStringConst(Node *node, const char *str);
 static bool rewrite_timestamp_walker(Node *node, void *context);
-static bool rewrite_timestamp_insert(InsertStmt *i_stmt, TSRewriteContext *ctx);
-static bool rewrite_timestamp_update(UpdateStmt *u_stmt, TSRewriteContext *ctx);
-static char *get_current_timestamp(POOL_CONNECTION_POOL *backend);
-static Node *makeTsExpr(TSRewriteContext *ctx);
-static A_Const *makeStringConstFromQuery(POOL_CONNECTION_POOL *backend, char *expression);
-bool raw_expression_tree_walker(Node *node, bool (*walker) (), void *context);
+static bool rewrite_timestamp_insert(InsertStmt *i_stmt, TSRewriteContext * ctx);
+static bool rewrite_timestamp_update(UpdateStmt *u_stmt, TSRewriteContext * ctx);
+static char *get_current_timestamp(POOL_CONNECTION_POOL * backend);
+static Node *makeTsExpr(TSRewriteContext * ctx);
+static TypeCast *makeTypeCastFromSvfOp(SQLValueFunctionOp op);
+static A_Const *makeStringConstFromQuery(POOL_CONNECTION_POOL * backend, char *expression);
+bool		raw_expression_tree_walker(Node *node, bool (*walker) (), void *context);
 
-POOL_RELCACHE	*ts_relcache;
+POOL_RELCACHE *ts_relcache;
 
 
 static void *
-ts_register_func(POOL_SELECT_RESULT *res)
+ts_register_func(POOL_SELECT_RESULT * res)
 {
 /* Number of result columns included in res */
 #define NUM_COLS		3
 
-	TSRel	*rel;
-	int		 i;
+	TSRel	   *rel;
+	int			i;
 
 	if (res->numrows == 0)
 		return NULL;
@@ -87,7 +93,7 @@ ts_register_func(POOL_SELECT_RESULT *res)
 
 	for (i = 0; i < res->numrows; i++)
 	{
-		int index = 0;
+		int			index = 0;
 
 		/* attribute name */
 		rel->attr[i].attrname = strdup(res->data[i * NUM_COLS + index]);
@@ -104,10 +110,10 @@ ts_register_func(POOL_SELECT_RESULT *res)
 		/* if timestamp is used in default value */
 		rel->attr[i].use_timestamp = *(res->data[i * NUM_COLS + index]) == 't';
 		ereport(DEBUG1,
-			(errmsg("timestamp register function"),
-				errdetail("attrname %s adsrc %s use_timestamp = %d",
-					   rel->attr[i].attrname, (rel->attr[i].adsrc? rel->attr[i].adsrc:"NULL"),
-					   rel->attr[i].use_timestamp)));
+				(errmsg("timestamp register function"),
+				 errdetail("attrname %s adsrc %s use_timestamp = %d",
+						   rel->attr[i].attrname, (rel->attr[i].adsrc ? rel->attr[i].adsrc : "NULL"),
+						   rel->attr[i].use_timestamp)));
 	}
 
 	/* num of attributes */
@@ -119,17 +125,20 @@ ts_register_func(POOL_SELECT_RESULT *res)
 static void *
 ts_unregister_func(void *data)
 {
-	TSRel	*rel = (TSRel *) data;
+	TSRel	   *rel = (TSRel *) data;
+
 	if (rel)
 		free(rel);
-	return (void *)0;
+	return (void *) 0;
 }
 
 
-static TSRel*
-relcache_lookup(TSRewriteContext *ctx)
+static TSRel *
+relcache_lookup(TSRewriteContext * ctx)
 {
-#define ATTRDEFQUERY "SELECT attname, d.adsrc, coalesce((d.adsrc LIKE '%%now()%%' OR d.adsrc LIKE '%%''now''::text%%')" \
+#define ATTRDEFQUERY "SELECT attname, d.adsrc, coalesce((d.adsrc LIKE '%%now()%%' OR d.adsrc LIKE '%%''now''::text%%' OR" \
+	" d.adsrc LIKE '%%CURRENT_TIMESTAMP%%' OR d.adsrc LIKE '%%CURRENT_TIME%%' OR d.adsrc LIKE '%%CURRENT_DATE%%' OR" \
+	" d.adsrc LIKE '%%LOCALTIME%%' OR d.adsrc LIKE '%%LOCALTIMESTAMP%%')" \
 	" AND (a.atttypid = 'timestamp'::regtype::oid OR" \
 	" a.atttypid = 'timestamp with time zone'::regtype::oid OR" \
 	" a.atttypid = 'date'::regtype::oid OR" \
@@ -141,7 +150,9 @@ relcache_lookup(TSRewriteContext *ctx)
 	" WHERE c.oid = a.attrelid AND a.attnum >= 1 AND a.attisdropped = 'f' AND c.relname = '%s'" \
 	" ORDER BY a.attnum"
 
-#define ATTRDEFQUERY2 "SELECT attname, d.adsrc, coalesce((d.adsrc LIKE '%%now()%%' OR d.adsrc LIKE '%%''now''::text%%')" \
+#define ATTRDEFQUERY2 "SELECT attname, d.adsrc, coalesce((d.adsrc LIKE '%%now()%%' OR d.adsrc LIKE '%%''now''::text%%' OR" \
+	" d.adsrc LIKE '%%CURRENT_TIMESTAMP%%' OR d.adsrc LIKE '%%CURRENT_TIME%%' OR d.adsrc LIKE '%%CURRENT_DATE%%' OR" \
+	" d.adsrc LIKE '%%LOCALTIME%%' OR d.adsrc LIKE '%%LOCALTIMESTAMP%%')" \
 	" AND (a.atttypid = 'timestamp'::regtype::oid OR" \
 	" a.atttypid = 'timestamp with time zone'::regtype::oid OR" \
 	" a.atttypid = 'date'::regtype::oid OR" \
@@ -153,7 +164,9 @@ relcache_lookup(TSRewriteContext *ctx)
 	" WHERE c.oid = a.attrelid AND a.attnum >= 1 AND a.attisdropped = 'f' AND c.oid = pgpool_regclass('%s')" \
 	" ORDER BY a.attnum"
 
-#define ATTRDEFQUERY3 "SELECT attname, d.adsrc, coalesce((d.adsrc LIKE '%%now()%%' OR d.adsrc LIKE '%%''now''::text%%')" \
+#define ATTRDEFQUERY3 "SELECT attname, d.adsrc, coalesce((d.adsrc LIKE '%%now()%%' OR d.adsrc LIKE '%%''now''::text%%' OR" \
+	" d.adsrc LIKE '%%CURRENT_TIMESTAMP%%' OR d.adsrc LIKE '%%CURRENT_TIME%%' OR d.adsrc LIKE '%%CURRENT_DATE%%' OR" \
+	" d.adsrc LIKE '%%LOCALTIME%%' OR d.adsrc LIKE '%%LOCALTIMESTAMP%%')" \
 	" AND (a.atttypid = 'timestamp'::regtype::oid OR" \
 	" a.atttypid = 'timestamp with time zone'::regtype::oid OR" \
 	" a.atttypid = 'date'::regtype::oid OR" \
@@ -165,8 +178,8 @@ relcache_lookup(TSRewriteContext *ctx)
 	" WHERE c.oid = a.attrelid AND a.attnum >= 1 AND a.attisdropped = 'f' AND c.oid = to_regclass('%s')" \
 	" ORDER BY a.attnum"
 
-	char *query;
-	char *table_name;
+	char	   *query;
+	char	   *table_name;
 
 	table_name = ctx->relname;
 
@@ -191,7 +204,7 @@ relcache_lookup(TSRewriteContext *ctx)
 		if (ts_relcache == NULL)
 		{
 			ereport(WARNING,
-				(errmsg("unable to create relcache")));
+					(errmsg("unable to create relcache")));
 			return NULL;
 		}
 	}
@@ -204,9 +217,9 @@ relcache_lookup(TSRewriteContext *ctx)
  * and add it into params list in context.
  */
 static Node *
-makeTsExpr(TSRewriteContext *ctx)
+makeTsExpr(TSRewriteContext * ctx)
 {
-	ParamRef		*param;
+	ParamRef   *param;
 
 	if (!ctx->rewrite_to_params)
 		return (Node *) ctx->ts_const;
@@ -221,7 +234,7 @@ makeTsExpr(TSRewriteContext *ctx)
 static bool
 isStringConst(Node *node, const char *str)
 {
-	A_Const		*a_const;
+	A_Const    *a_const;
 	Value		val;
 
 	if (!IsA(node, A_Const))
@@ -240,7 +253,7 @@ isStringConst(Node *node, const char *str)
 bool
 isSystemType(Node *node, const char *name)
 {
-	TypeName	*typename;
+	TypeName   *typename;
 
 	if (!IsA(node, TypeName))
 		return false;
@@ -258,7 +271,7 @@ isSystemType(Node *node, const char *name)
 static bool
 isSystemTypeCast(Node *node, const char *name)
 {
-	TypeCast	*typecast;
+	TypeCast   *typecast;
 
 	if (!IsA(node, TypeCast))
 		return false;
@@ -273,7 +286,7 @@ isSystemTypeCast(Node *node, const char *name)
 static bool
 rewrite_timestamp_walker(Node *node, void *context)
 {
-	TSRewriteContext	*ctx = (TSRewriteContext *) context;
+	TSRewriteContext *ctx = (TSRewriteContext *) context;
 
 	if (node == NULL)
 		return false;
@@ -282,8 +295,8 @@ rewrite_timestamp_walker(Node *node, void *context)
 	{
 		case T_InsertStmt:
 			{
-				TSRewriteContext	*ctx = (TSRewriteContext *) context;
-				char *relname_old;
+				TSRewriteContext *ctx = (TSRewriteContext *) context;
+				char	   *relname_old;
 				InsertStmt *i_stmt = (InsertStmt *) node;
 
 				relname_old = ctx->relname;
@@ -291,12 +304,12 @@ rewrite_timestamp_walker(Node *node, void *context)
 				rewrite_timestamp_insert(i_stmt, ctx);
 				ctx->relname = relname_old;
 			}
-			/* tree walker is called in rewrite_timestamp_insert() */ 
+			/* tree walker is called in rewrite_timestamp_insert() */
 			return false;
 		case T_UpdateStmt:
 			{
-				TSRewriteContext	*ctx = (TSRewriteContext *) context;
-				char *relname_old;
+				TSRewriteContext *ctx = (TSRewriteContext *) context;
+				char	   *relname_old;
 				UpdateStmt *u_stmt = (UpdateStmt *) node;
 
 				relname_old = ctx->relname;
@@ -304,41 +317,94 @@ rewrite_timestamp_walker(Node *node, void *context)
 				rewrite_timestamp_update(u_stmt, ctx);
 				ctx->relname = relname_old;
 			}
-			/* tree walker is called in rewrite_timestamp_update() */ 
+			/* tree walker is called in rewrite_timestamp_update() */
 			return false;
 		case T_FuncCall:
 			{
 				/* `now()' FuncCall */
-				FuncCall	*fcall = (FuncCall *) node;
+				FuncCall   *fcall = (FuncCall *) node;
 
 				if ((list_length(fcall->funcname) == 1 &&
-			 		strcmp("now", strVal(linitial(fcall->funcname))) == 0) ||
+					 strcmp("now", strVal(linitial(fcall->funcname))) == 0) ||
 					(list_length(fcall->funcname) == 2 &&
-			 		strcmp("pg_catalog", strVal(linitial(fcall->funcname))) == 0 &&
-			 		strcmp("now", strVal(lsecond(fcall->funcname))) == 0))
+					 strcmp("pg_catalog", strVal(linitial(fcall->funcname))) == 0 &&
+					 strcmp("now", strVal(lsecond(fcall->funcname))) == 0))
 				{
-					TypeCast	*tc = makeNode(TypeCast);
+					TypeCast   *tc = makeNode(TypeCast);
+
 					tc->arg = makeTsExpr(ctx);
 					tc->typeName = SystemTypeName("text");
-	
+
 					fcall->funcname = SystemFuncName("timestamptz");
 					fcall->args = list_make1(tc);
 					ctx->rewrite = true;
 				}
 			}
 			break;
+		case T_SQLValueFunction:
+			{
+
+				/*
+				 * CURRENT_DATE, CURRENT_TIME, LOCALTIMESTAMP, LOCALTIME etc.
+				 * From PostgreSQL 10, timestamp conversion is changed and
+				 * SQLValueFunction is added. But Pgpool-II should use the old
+				 * timestamp translate like "'now'::text::date". So we have to
+				 * cast the node to TypeCast first.
+				 */
+				SQLValueFunction *svf = (SQLValueFunction *) node;
+
+				if (svf->op == SVFOP_CURRENT_DATE ||
+					svf->op == SVFOP_CURRENT_TIME ||
+					svf->op == SVFOP_CURRENT_TIME_N ||
+					svf->op == SVFOP_CURRENT_TIMESTAMP ||
+					svf->op == SVFOP_CURRENT_TIMESTAMP_N ||
+					svf->op == SVFOP_LOCALTIME ||
+					svf->op == SVFOP_LOCALTIME_N ||
+					svf->op == SVFOP_LOCALTIMESTAMP ||
+					svf->op == SVFOP_LOCALTIMESTAMP_N)
+				{
+
+					TypeCast   *tc,
+							   *tc1;
+
+					tc1 = makeTypeCastFromSvfOp(svf->op);
+
+					tc = (TypeCast *) node;
+					tc->arg = tc1->arg;
+					tc->typeName = (TypeName *) copyObject(tc1->typeName);
+					tc->location = -1;
+
+					node->type = T_TypeCast;
+
+					/*
+					 * rewrite `'now'::timestamp' and `'now'::text::timestamp'
+					 * both
+					 */
+					if (isSystemTypeCast(tc->arg, "text"))
+						tc = (TypeCast *) tc->arg;
+
+					if (isStringConst(tc->arg, "now"))
+					{
+						tc->arg = (Node *) makeTsExpr(ctx);
+						ctx->rewrite = true;
+					}
+				}
+			}
+			break;
 		case T_TypeCast:
 			{
-				/* CURRENT_DATE, CURRENT_TIME, LOCALTIMESTAMP, LOCALTIME etc.*/
-				TypeCast	*tc = (TypeCast *) node;
+				TypeCast   *tc = (TypeCast *) node;
 
 				if ((isSystemType((Node *) tc->typeName, "date") ||
-			 		isSystemType((Node *) tc->typeName, "timestamp") ||
-			 		isSystemType((Node *) tc->typeName, "timestamptz") ||
-			 		isSystemType((Node *) tc->typeName, "time") ||
-			 		isSystemType((Node *) tc->typeName, "timetz")))
+					 isSystemType((Node *) tc->typeName, "timestamp") ||
+					 isSystemType((Node *) tc->typeName, "timestamptz") ||
+					 isSystemType((Node *) tc->typeName, "time") ||
+					 isSystemType((Node *) tc->typeName, "timetz")))
 				{
-					/* rewrite `'now'::timestamp' and `'now'::text::timestamp' both */
+					/*
+					 * rewrite `'now'::timestamp' and `'now'::text::timestamp'
+					 * both
+					 */
 					if (isSystemTypeCast(tc->arg, "text"))
 						tc = (TypeCast *) tc->arg;
 
@@ -352,7 +418,7 @@ rewrite_timestamp_walker(Node *node, void *context)
 			break;
 		case T_ParamRef:
 			{
-				ParamRef	*param = (ParamRef *) node;
+				ParamRef   *param = (ParamRef *) node;
 
 				/* count how many params in original query */
 				if (ctx->num_params < param->number)
@@ -371,10 +437,10 @@ rewrite_timestamp_walker(Node *node, void *context)
  * Get `now()' from MASTER node
  */
 static char *
-get_current_timestamp(POOL_CONNECTION_POOL *backend)
+get_current_timestamp(POOL_CONNECTION_POOL * backend)
 {
 	POOL_SELECT_RESULT *res;
-	static char		timestamp[64];
+	static char timestamp[64];
 
 	do_query(MASTER(backend), "SELECT now()", &res, MAJOR(backend));
 
@@ -395,35 +461,34 @@ get_current_timestamp(POOL_CONNECTION_POOL *backend)
  * rewrite InsertStmt
  */
 static bool
-rewrite_timestamp_insert(InsertStmt *i_stmt, TSRewriteContext *ctx)
+rewrite_timestamp_insert(InsertStmt *i_stmt, TSRewriteContext * ctx)
 {
-	int		 i;
-	bool	 rewrite = false;
-	TSRel	*relcache;
+	int			i;
+	bool		rewrite = false;
+	TSRel	   *relcache;
 
 	raw_expression_tree_walker(
-			(Node *) i_stmt->withClause,
-			rewrite_timestamp_walker, (void *) ctx);
+							   (Node *) i_stmt->withClause,
+							   rewrite_timestamp_walker, (void *) ctx);
 
 	raw_expression_tree_walker(
-			(Node *) i_stmt->onConflictClause,
-			rewrite_timestamp_walker, (void *) ctx);
+							   (Node *) i_stmt->onConflictClause,
+							   rewrite_timestamp_walker, (void *) ctx);
 
 	raw_expression_tree_walker(
-			(Node *) i_stmt->returningList,
-			rewrite_timestamp_walker, (void *) ctx);
+							   (Node *) i_stmt->returningList,
+							   rewrite_timestamp_walker, (void *) ctx);
 
 	rewrite = ctx->rewrite;
 
 	/*
-	 * INSERT INTO rel DEFAULT VALUES
-	 * rewrite to:
-	 * INSERT INTO rel VALUES (DEFAULT, '2009-..',...)
+	 * INSERT INTO rel DEFAULT VALUES rewrite to: INSERT INTO rel VALUES
+	 * (DEFAULT, '2009-..',...)
 	 */
 	if (i_stmt->selectStmt == NULL)
 	{
-		List		*values = NIL;
-		SelectStmt	*selectStmt = makeNode(SelectStmt);
+		List	   *values = NIL;
+		SelectStmt *selectStmt = makeNode(SelectStmt);
 
 		relcache = relcache_lookup(ctx);
 		if (relcache == NULL)
@@ -451,15 +516,17 @@ rewrite_timestamp_insert(InsertStmt *i_stmt, TSRewriteContext *ctx)
 	}
 	else if (IsA(i_stmt->selectStmt, SelectStmt))
 	{
-		SelectStmt	*selectStmt = (SelectStmt *) i_stmt->selectStmt;
-		ListCell	*lc_row, *lc_val, *lc_col;
+		SelectStmt *selectStmt = (SelectStmt *) i_stmt->selectStmt;
+		ListCell   *lc_row,
+				   *lc_val,
+				   *lc_col;
 
 		/*
 		 * Rewrite `now()' call to timestamp literal.
 		 */
 		raw_expression_tree_walker(
-				(Node *) selectStmt,
-				rewrite_timestamp_walker, (void *) ctx);
+								   (Node *) selectStmt,
+								   rewrite_timestamp_walker, (void *) ctx);
 
 		rewrite = ctx->rewrite;
 
@@ -478,23 +545,18 @@ rewrite_timestamp_insert(InsertStmt *i_stmt, TSRewriteContext *ctx)
 			/*
 			 * INSERT INTO rel VALUES (...)
 			 *
-			 * CREATE TABLE r1 (
-			 *   i1 int,
-			 *   t1 timestamp default now(),
-			 *   i2 int,
-			 *   t2 timestamp default now(),
-			 * )
+			 * CREATE TABLE r1 ( i1 int, t1 timestamp default now(), i2 int,
+			 * t2 timestamp default now(), )
 			 *
-			 * INSERT INTO r1 VALUES (1, DEFAULT);
-			 * rewrite to:
-			 * INSERT INTO r1 VALUES (1, '20xx-xx-xx...', DEFAULT, '2..')
+			 * INSERT INTO r1 VALUES (1, DEFAULT); rewrite to: INSERT INTO r1
+			 * VALUES (1, '20xx-xx-xx...', DEFAULT, '2..')
 			 */
-			foreach (lc_row, selectStmt->valuesLists)
+			foreach(lc_row, selectStmt->valuesLists)
 			{
-				List		*values = lfirst(lc_row);
+				List	   *values = lfirst(lc_row);
 
 				i = 0;
-				foreach (lc_val, values)
+				foreach(lc_val, values)
 				{
 					if (relcache->attr[i].use_timestamp == true && IsA(lfirst(lc_val), SetToDefault))
 					{
@@ -529,21 +591,21 @@ rewrite_timestamp_insert(InsertStmt *i_stmt, TSRewriteContext *ctx)
 			/*
 			 * INSERT INTO rel(col1, col2) VALUES (val, val2)
 			 *
-			 * if timestamp column is not given by column list
-			 * add colname to column list and add timestamp to values list.
+			 * if timestamp column is not given by column list add colname to
+			 * column list and add timestamp to values list.
 			 */
 			int			appended_columns = 0;
-			int			*appended_columns_list;
-			ResTarget	*col;
+			int		   *appended_columns_list;
+			ResTarget  *col;
 
-			appended_columns_list = (int *)malloc(sizeof(int)*relcache->relnatts);
+			appended_columns_list = (int *) malloc(sizeof(int) * relcache->relnatts);
 
 			for (i = 0; i < relcache->relnatts; i++)
 			{
 				if (relcache->attr[i].use_timestamp == false)
 					continue;
 
-				foreach (lc_col, i_stmt->cols)
+				foreach(lc_col, i_stmt->cols)
 				{
 					col = lfirst(lc_col);
 
@@ -551,7 +613,7 @@ rewrite_timestamp_insert(InsertStmt *i_stmt, TSRewriteContext *ctx)
 						break;
 				}
 
-				/* If columns are not found in query, append these.*/
+				/* If columns are not found in query, append these. */
 				if (lc_col == NULL)
 				{
 					rewrite = true;
@@ -564,12 +626,12 @@ rewrite_timestamp_insert(InsertStmt *i_stmt, TSRewriteContext *ctx)
 				}
 			}
 
-			foreach (lc_row, selectStmt->valuesLists)
+			foreach(lc_row, selectStmt->valuesLists)
 			{
-				List		*values = lfirst(lc_row);
+				List	   *values = lfirst(lc_row);
 
 				/* replace DEFAULT in VALUES to literal */
-				forboth (lc_col, i_stmt->cols, lc_val, values)
+				forboth(lc_col, i_stmt->cols, lc_val, values)
 				{
 					col = lfirst(lc_col);
 					for (i = 0; i < relcache->relnatts; i++)
@@ -610,40 +672,40 @@ rewrite_timestamp_insert(InsertStmt *i_stmt, TSRewriteContext *ctx)
  * rewrite UpdateStmt
  */
 static bool
-rewrite_timestamp_update(UpdateStmt *u_stmt, TSRewriteContext *ctx)
+rewrite_timestamp_update(UpdateStmt *u_stmt, TSRewriteContext * ctx)
 {
-	TSRel			*relcache = NULL;
-	ListCell		*lc;
-	bool			 rewrite;
+	TSRel	   *relcache = NULL;
+	ListCell   *lc;
+	bool		rewrite;
 
 	/* rewrite "update ... set col1 = now()" */
 	raw_expression_tree_walker(
-			(Node *) u_stmt->targetList,
-			rewrite_timestamp_walker, (void *) ctx);
+							   (Node *) u_stmt->targetList,
+							   rewrite_timestamp_walker, (void *) ctx);
 
 	raw_expression_tree_walker(
-			(Node *) u_stmt->whereClause,
-			rewrite_timestamp_walker, (void *) ctx);
+							   (Node *) u_stmt->whereClause,
+							   rewrite_timestamp_walker, (void *) ctx);
 
 	raw_expression_tree_walker(
-			(Node *) u_stmt->fromClause,
-			rewrite_timestamp_walker, (void *) ctx);
+							   (Node *) u_stmt->fromClause,
+							   rewrite_timestamp_walker, (void *) ctx);
 
 	raw_expression_tree_walker(
-			(Node *) u_stmt->withClause,
-			rewrite_timestamp_walker, (void *) ctx);
+							   (Node *) u_stmt->withClause,
+							   rewrite_timestamp_walker, (void *) ctx);
 
 	raw_expression_tree_walker(
-			(Node *) u_stmt->returningList,
-			rewrite_timestamp_walker, (void *) ctx);
+							   (Node *) u_stmt->returningList,
+							   rewrite_timestamp_walker, (void *) ctx);
 
 	rewrite = ctx->rewrite;
 
 	/* rewrite "update ... set col1 = default" */
-	foreach (lc, u_stmt->targetList)
+	foreach(lc, u_stmt->targetList)
 	{
-		ResTarget	*res = (ResTarget *) lfirst(lc);
-		int			 i;
+		ResTarget  *res = (ResTarget *) lfirst(lc);
+		int			i;
 
 		if (IsA(res->val, SetToDefault))
 		{
@@ -660,7 +722,7 @@ rewrite_timestamp_update(UpdateStmt *u_stmt, TSRewriteContext *ctx)
 						if (ctx->rewrite_to_params)
 							res->val = (Node *) makeTsExpr(ctx);
 						else
-							res->val = (Node *)makeStringConstFromQuery(ctx->backend, relcache->attr[i].adsrc);
+							res->val = (Node *) makeStringConstFromQuery(ctx->backend, relcache->attr[i].adsrc);
 						rewrite = true;
 					}
 					break;
@@ -678,14 +740,14 @@ rewrite_timestamp_update(UpdateStmt *u_stmt, TSRewriteContext *ctx)
  * returns query string as palloced string, or NULL if not to need rewrite.
  */
 char *
-rewrite_timestamp(POOL_CONNECTION_POOL *backend, Node *node,
-				  bool rewrite_to_params, POOL_SENT_MESSAGE *message)
+rewrite_timestamp(POOL_CONNECTION_POOL * backend, Node *node,
+				  bool rewrite_to_params, POOL_SENT_MESSAGE * message)
 {
-	TSRewriteContext	ctx;
-	Node			*stmt;
-	bool			 rewrite = false;
-	char			*timestamp;
-	char			*rewrite_query;
+	TSRewriteContext ctx;
+	Node	   *stmt;
+	bool		rewrite = false;
+	char	   *timestamp;
+	char	   *rewrite_query;
 
 	if (node == NULL)
 		return NULL;
@@ -710,7 +772,7 @@ rewrite_timestamp(POOL_CONNECTION_POOL *backend, Node *node,
 		stmt = ((PrepareStmt *) node)->query;
 		ctx.rewrite_to_params = true;
 	}
-	else if (IsA(node, CopyStmt) && ((CopyStmt *) node)->query != NULL)
+	else if (IsA(node, CopyStmt) &&((CopyStmt *) node)->query != NULL)
 		stmt = ((CopyStmt *) node)->query;
 	else
 		stmt = node;
@@ -719,33 +781,36 @@ rewrite_timestamp(POOL_CONNECTION_POOL *backend, Node *node,
 	if (IsA(stmt, InsertStmt))
 	{
 		InsertStmt *i_stmt = (InsertStmt *) stmt;
+
 		ctx.relname = make_table_name_from_rangevar(i_stmt->relation);
 		rewrite = rewrite_timestamp_insert(i_stmt, &ctx);
 	}
 	else if (IsA(stmt, UpdateStmt))
 	{
 		UpdateStmt *u_stmt = (UpdateStmt *) stmt;
+
 		ctx.relname = make_table_name_from_rangevar(u_stmt->relation);
 		rewrite = rewrite_timestamp_update(u_stmt, &ctx);
 	}
 	else if (IsA(stmt, DeleteStmt))
 	{
 		DeleteStmt *d_stmt = (DeleteStmt *) stmt;
-		raw_expression_tree_walker(
-				(Node *) d_stmt->usingClause,
-				rewrite_timestamp_walker, (void *) &ctx);
 
 		raw_expression_tree_walker(
-				(Node *) d_stmt->whereClause,
-				rewrite_timestamp_walker, (void *) &ctx);
+								   (Node *) d_stmt->usingClause,
+								   rewrite_timestamp_walker, (void *) &ctx);
 
 		raw_expression_tree_walker(
-				(Node *) d_stmt->returningList,
-				rewrite_timestamp_walker, (void *) &ctx);
+								   (Node *) d_stmt->whereClause,
+								   rewrite_timestamp_walker, (void *) &ctx);
 
 		raw_expression_tree_walker(
-				(Node *) d_stmt->withClause,
-				rewrite_timestamp_walker, (void *) &ctx);
+								   (Node *) d_stmt->returningList,
+								   rewrite_timestamp_walker, (void *) &ctx);
+
+		raw_expression_tree_walker(
+								   (Node *) d_stmt->withClause,
+								   rewrite_timestamp_walker, (void *) &ctx);
 
 		rewrite = ctx.rewrite;
 	}
@@ -755,17 +820,18 @@ rewrite_timestamp(POOL_CONNECTION_POOL *backend, Node *node,
 
 		/* rewrite params */
 		raw_expression_tree_walker(
-				(Node *) e_stmt->params,
-				rewrite_timestamp_walker, (void *) &ctx);
+								   (Node *) e_stmt->params,
+								   rewrite_timestamp_walker, (void *) &ctx);
 
 		rewrite = ctx.rewrite;
 
-		/* add timestamp value into params list.
-		 * The number of params is stored in message
+		/*
+		 * add timestamp value into params list. The number of params is
+		 * stored in message
 		 */
 		if (message)
 		{
-			int		i;
+			int			i;
 
 			for (i = 0; i < message->num_tsparams; i++)
 			{
@@ -777,8 +843,9 @@ rewrite_timestamp(POOL_CONNECTION_POOL *backend, Node *node,
 	else
 		;
 
-	/* PREPARE, EXECUTE or Parse:
-	 * save the number of parameters in the original query
+	/*
+	 * PREPARE, EXECUTE or Parse: save the number of parameters in the
+	 * original query
 	 */
 	if (message)
 		message->query_context->num_original_params = ctx.num_params;
@@ -787,18 +854,19 @@ rewrite_timestamp(POOL_CONNECTION_POOL *backend, Node *node,
 	if (!rewrite)
 		return NULL;
 
-	/* PREPARE or Parse:
-	 * handle additinal parameters for timestamps
+	/*
+	 * PREPARE or Parse: handle additinal parameters for timestamps
 	 */
 	if (ctx.rewrite_to_params && message)
 	{
-		ListCell	*lc;
-		int			 num = ctx.num_params + 1;
+		ListCell   *lc;
+		int			num = ctx.num_params + 1;
 
 		/* renumber params */
-		foreach (lc, ctx.params)
+		foreach(lc, ctx.params)
 		{
-			ParamRef		*param = (ParamRef *) lfirst(lc);
+			ParamRef   *param = (ParamRef *) lfirst(lc);
+
 			param->number = num++;
 		}
 
@@ -808,12 +876,12 @@ rewrite_timestamp(POOL_CONNECTION_POOL *backend, Node *node,
 		/* add type names of timestamptz into argtypes */
 		if (IsA(node, PrepareStmt))
 		{
-			int				 i;
-			PrepareStmt		*p_stmt = (PrepareStmt *) node;
+			int			i;
+			PrepareStmt *p_stmt = (PrepareStmt *) node;
 
 			for (i = 0; i < message->num_tsparams; i++)
 				p_stmt->argtypes =
-				   	lappend(p_stmt->argtypes, SystemTypeName("timestamptz"));
+					lappend(p_stmt->argtypes, SystemTypeName("timestamptz"));
 		}
 	}
 	else
@@ -838,36 +906,38 @@ rewrite_timestamp(POOL_CONNECTION_POOL *backend, Node *node,
  * rewrite Bind message to add parameter values
  */
 char *
-bind_rewrite_timestamp(POOL_CONNECTION_POOL *backend,
-					   POOL_SENT_MESSAGE *message,
+bind_rewrite_timestamp(POOL_CONNECTION_POOL * backend,
+					   POOL_SENT_MESSAGE * message,
 					   const char *orig_msg, int *len)
 {
-	int16		 tmp2,
-				 num_params,
-				 num_formats;
-	int32		 tmp4;
-	int			 i,
-				 ts_len,
-				 copy_len,
-				 num_org_params;
-	const char	*copy_from;
-	char		*ts,
-				*copy_to,
-				*new_msg;
+	int16		tmp2,
+				num_params,
+				num_formats,
+				num_formats_new;
+	int32		tmp4;
+	int			i,
+				ts_len,
+				copy_len,
+				num_org_params;
+	const char *copy_from;
+	char	   *ts,
+			   *copy_to,
+			   *new_msg;
 
 #ifdef TIMESTAMPDEBUG
 	fprintf(stderr, "message length:%d\n", *len);
-	for(i=0;i<*len;i++)
+	for (i = 0; i < *len; i++)
 	{
 		fprintf(stderr, "%02x ", orig_msg[i]);
 	}
+	fprintf(stderr, "\n");
 #endif
 
 	ts = get_current_timestamp(backend);
 	if (ts == NULL)
 	{
 		ereport(WARNING,
-			(errmsg("bind rewrite timestamp failed, unable to get current timestamp")));
+				(errmsg("bind rewrite timestamp failed, unable to get current timestamp")));
 		return NULL;
 	}
 
@@ -878,53 +948,77 @@ bind_rewrite_timestamp(POOL_CONNECTION_POOL *backend,
 	/* allocate extra memory for parameter formats */
 	num_org_params = message->query_context->num_original_params;
 	new_msg = copy_to = (char *) palloc(*len + sizeof(int16) * (message->num_tsparams + num_org_params));
+
 	copy_from = orig_msg;
 
-	/* portal_name */
+	/* 1. portal_name */
 	copy_len = strlen(copy_from) + 1;
 	memcpy(copy_to, copy_from, copy_len);
-	copy_to += copy_len; copy_from += copy_len;
+	copy_to += copy_len;
+	copy_from += copy_len;
 
-	/* stmt_name */
+	/* 2. stmt_name */
 	copy_len = strlen(copy_from) + 1;
 	memcpy(copy_to, copy_from, copy_len);
-	copy_to += copy_len; copy_from += copy_len;
+	copy_to += copy_len;
+	copy_from += copy_len;
 
-	/* format code */
+	/* 3. format codes */
+
+	/* 3.1. the number of format codes */
 	memcpy(&tmp2, copy_from, sizeof(int16));
-	copy_len = sizeof(int16);
-	tmp2 = num_formats = ntohs(tmp2);
+	num_formats = ntohs(tmp2);
 
-	if (num_formats >= 1)
+	if (num_formats == 0)
 	{
-		/* this means the specified format code is applied all original parameters */
+		/*
+		 * If num_formats is 0, the original message has no parameters or the parameter formats are all text,
+		 * so we don't need additional format codes since timestamp parametes use text as its format.
+		 */
+		num_formats_new = 0;
+	}
+	else
+	{
+		/* If num formats is 1, this means the specified format code is applied for all original parameters,
+		 * so enlarge message length to specify format codes for each of original paramters. */
 		if (num_formats == 1)
-		{
 			*len += (num_org_params - 1) * sizeof(int16);
-			tmp2 += num_org_params - 1;
-		}
 
 		/* enlarge message length for timestamp parameter's formats */
 		*len += message->num_tsparams * sizeof(int16);
-		tmp2 += message->num_tsparams;
+		num_formats_new = num_org_params + message->num_tsparams;
 	}
 
-	tmp2 = htons(tmp2);
-	memcpy(copy_to, &tmp2, copy_len);	/* copy number of format codes */
-	copy_to += copy_len; copy_from += copy_len;
-
-	copy_len = num_formats * sizeof(int16);
-
-	memcpy(copy_to, copy_from, copy_len);		/* copy format codes */
-	copy_to += copy_len; copy_from += copy_len;
+	/* copy the number of format codes */
+	tmp2 = htons(num_formats_new);
+	copy_len = sizeof(int16);
+	memcpy(copy_to, &tmp2, copy_len);
+	copy_to += copy_len;
+	copy_from += copy_len;
 
+	/* 3.2. the format codes */
 	if (num_formats >= 1)
 	{
-		/* copy the specified format code as numbers of original parameters */
+		/* If num_formats is 1, copy the specified format code as numbers of original parameters */
 		if (num_formats == 1)
 		{
-			memcpy(copy_to, copy_from, (num_org_params - 1) * sizeof(int16));
-			copy_to += (num_org_params - 1) * sizeof(int16);
+			int16	org_format_code;
+			memcpy(&org_format_code, copy_from, sizeof(int16));
+			copy_from += copy_len;
+
+			for (i = 0; i < num_org_params; i++)
+			{
+				memcpy(copy_to, &org_format_code, sizeof(int16));
+				copy_to += sizeof(int16);
+			}
+		}
+		/* otherwize, copy the original format codes as they are*/
+		else
+		{
+			copy_len = num_formats * sizeof(int16);
+			memcpy(copy_to, copy_from, copy_len);
+			copy_to += copy_len;
+			copy_from += copy_len;
 		}
 
 		/* set additional format codes to zero(text) */
@@ -932,15 +1026,18 @@ bind_rewrite_timestamp(POOL_CONNECTION_POOL *backend,
 		copy_to += sizeof(int16) * message->num_tsparams;
 	}
 
+	/* 4. parameters */
+
 	/* num params */
 	memcpy(&tmp2, copy_from, sizeof(int16));
 	copy_len = sizeof(int16);
 	num_params = ntohs(tmp2);
 	tmp2 = htons(num_params + message->num_tsparams);
 	memcpy(copy_to, &tmp2, sizeof(int16));
-	copy_to += copy_len; copy_from += copy_len;
+	copy_to += copy_len;
+	copy_from += copy_len;
 
-	/* params */
+	/* original params */
 	copy_len = 0;
 	for (i = 0; i < num_params; i++)
 	{
@@ -948,8 +1045,9 @@ bind_rewrite_timestamp(POOL_CONNECTION_POOL *backend,
 		tmp4 = ntohl(tmp4);		/* param length */
 		copy_len += sizeof(int32);
 
-		/* If param length is -1, it indicates that the value is NULL
-		 * and we don't have value slot. So we don't add up copy_len.
+		/*
+		 * If param length is -1, it indicates that the value is NULL and we
+		 * don't have value slot. So we don't add up copy_len.
 		 */
 		if (tmp4 > 0)
 		{
@@ -957,8 +1055,10 @@ bind_rewrite_timestamp(POOL_CONNECTION_POOL *backend,
 		}
 	}
 	memcpy(copy_to, copy_from, copy_len);
-	copy_to += copy_len; copy_from += copy_len;
+	copy_to += copy_len;
+	copy_from += copy_len;
 
+	/* timestamp params */
 	tmp4 = htonl(ts_len);
 	for (i = 0; i < message->num_tsparams; i++)
 	{
@@ -968,23 +1068,33 @@ bind_rewrite_timestamp(POOL_CONNECTION_POOL *backend,
 		copy_to += ts_len;
 	}
 
-	/* result format code */
+	/* 5. result format code */
 	memcpy(&tmp2, copy_from, sizeof(int16));
 	copy_len = sizeof(int16);
 	copy_len += sizeof(int16) * ntohs(tmp2);
 	memcpy(copy_to, copy_from, copy_len);
 
+#ifdef TIMESTAMPDEBUG
+	fprintf(stderr, "message length:%d\n", *len);
+	for(i = 0; i < *len; i++)
+	{
+		fprintf(stderr, "%02x ", new_msg[i]);
+	}
+	fprintf(stderr, "\n");
+#endif
+
 	return new_msg;
 }
 
 /* make A_Const of T_String from "SELECT <expression>"*/
-static A_Const *makeStringConstFromQuery(POOL_CONNECTION_POOL *backend, char *expression)
+static A_Const *
+makeStringConstFromQuery(POOL_CONNECTION_POOL * backend, char *expression)
 {
-	A_Const *con;
+	A_Const    *con;
 	POOL_SELECT_RESULT *res;
-	char query[1024];
-	int len;
-	char *str;
+	char		query[1024];
+	int			len;
+	char	   *str;
 
 	snprintf(query, sizeof(query), "SELECT %s", expression);
 	do_query(MASTER(backend), query, &res, MAJOR(backend));
@@ -1006,6 +1116,69 @@ static A_Const *makeStringConstFromQuery(POOL_CONNECTION_POOL *backend, char *ex
 	return con;
 }
 
+static TypeCast *
+makeTypeCastFromSvfOp(SQLValueFunctionOp op)
+{
+	TypeName   *typename;
+	Node	   *n;
+	int			location;
+
+	switch (op)
+	{
+		case SVFOP_CURRENT_DATE:
+			typename = SystemTypeName("date");
+			location = 0;
+			break;
+		case SVFOP_CURRENT_TIME:
+			typename = SystemTypeName("timetz");
+			location = 0;
+			break;
+		case SVFOP_CURRENT_TIME_N:
+			typename = SystemTypeName("timetz");
+			typename->typmods = lcons(makeIntConst(0, 0), NIL);
+			location = -3;
+			break;
+		case SVFOP_CURRENT_TIMESTAMP:
+			typename = SystemTypeName("timestamptz");
+			location = 0;
+			break;
+		case SVFOP_CURRENT_TIMESTAMP_N:
+			typename = SystemTypeName("timestamptz");
+			typename->typmods = lcons(makeIntConst(0, 0), NIL);
+			location = -3;
+			break;
+		case SVFOP_LOCALTIME:
+			typename = SystemTypeName("time");
+			location = 0;
+			break;
+		case SVFOP_LOCALTIME_N:
+			typename = SystemTypeName("time");
+			typename->typmods = lcons(makeIntConst(0, 0), NIL);
+			location = -3;
+			break;
+		case SVFOP_LOCALTIMESTAMP:
+			typename = SystemTypeName("timestamp");
+			location = 0;
+			break;
+		case SVFOP_LOCALTIMESTAMP_N:
+			typename = SystemTypeName("timestamp");;
+			typename->typmods = lcons(makeIntConst(0, 0), NIL);
+			location = -3;
+			break;
+		case SVFOP_CURRENT_ROLE:
+		case SVFOP_CURRENT_USER:
+		case SVFOP_USER:
+		case SVFOP_SESSION_USER:
+		case SVFOP_CURRENT_CATALOG:
+		case SVFOP_CURRENT_SCHEMA:
+			break;
+	}
+
+	n = makeStringConstCast("now", -1, SystemTypeName("text"));
+
+	return (TypeCast *) makeTypeCast(n, typename, location);
+}
+
 /* from nodeFuncs.c start */
 
 /*
@@ -1036,9 +1209,10 @@ raw_expression_tree_walker(Node *node,
 		return false;
 
 	/* Guard against stack overflow due to overly complex expressions */
+
 	/*
-	check_stack_depth();
-	*/
+	 * check_stack_depth();
+	 */
 
 	switch (nodeTag(node))
 	{
@@ -1440,13 +1614,13 @@ raw_expression_tree_walker(Node *node,
 		case T_CommonTableExpr:
 			return walker(((CommonTableExpr *) node)->ctequery, context);
 		default:
+
 			/*
-			elog(ERROR, "unrecognized node type: %d",
-				 (int) nodeTag(node));
-				 */
+			 * elog(ERROR, "unrecognized node type: %d", (int) nodeTag(node));
+			 */
 			break;
 	}
 	return false;
 }
-/* from nodeFuncs.c end */
 
+/* from nodeFuncs.c end */
