@@ -3,7 +3,7 @@
  * pgpool_adm.c
  *
  *
- * Copyright (c) 2002-2015, PostgreSQL Global Development Group
+ * Copyright (c) 2002-2018, PostgreSQL Global Development Group
  *
  * Author: Jehan-Guillaume (ioguix) de Rorthais <jgdr@dalibo.com>
  *
@@ -17,8 +17,9 @@
 #include "utils/builtins.h"
 #include "foreign/foreign.h"
 #include "nodes/pg_list.h"
+#include "utils/timestamp.h"
 
-/* 
+/*
  * PostgreSQL 9.3 or later requires htup_details.h to get the definition of
  * heap_form_tuple
  */
@@ -27,6 +28,7 @@
 #endif
 
 #include <unistd.h>
+#include <time.h>
 
 #include "catalog/pg_type.h"
 #include "funcapi.h"
@@ -34,23 +36,24 @@
 #include "pgpool_adm.h"
 
 
-static PCPConnInfo *connect_to_server(char* host, int port, char* user, char* pass);
-static PCPConnInfo *connect_to_server_from_foreign_server(char * name);
+static PCPConnInfo * connect_to_server(char *host, int port, char *user, char *pass);
+static PCPConnInfo * connect_to_server_from_foreign_server(char *name);
 
 
 /**
  * Wrapper around pcp_connect
  * pcp_conninfo: pcpConninfo structure having pcp connection properties
  */
-static PCPConnInfo*
-connect_to_server(char* host, int port, char* user, char* pass)
+static PCPConnInfo *
+connect_to_server(char *host, int port, char *user, char *pass)
 {
-	PCPConnInfo* pcpConnInfo;
+	PCPConnInfo *pcpConnInfo;
+
 	pcpConnInfo = pcp_connect(host, port, user, pass, NULL);
 	if (PCPConnectionStatus(pcpConnInfo) != PCP_CONNECTION_OK)
-		ereport(ERROR,(0,
-					   errmsg("connection to PCP server failed."),
-					   errdetail("%s\n",pcp_get_last_error(pcpConnInfo)?pcp_get_last_error(pcpConnInfo):"unknown reason")));
+		ereport(ERROR, (0,
+						errmsg("connection to PCP server failed."),
+						errdetail("%s\n", pcp_get_last_error(pcpConnInfo) ? pcp_get_last_error(pcpConnInfo) : "unknown reason")));
 
 	return pcpConnInfo;
 }
@@ -59,25 +62,29 @@ connect_to_server(char* host, int port, char* user, char* pass)
  * Returns a pcpConninfo structure filled from a foreign server
  * name: the name of the foreign server
  */
-static PCPConnInfo*
-connect_to_server_from_foreign_server(char * name)
+static PCPConnInfo *
+connect_to_server_from_foreign_server(char *name)
 {
-	Oid userid = GetUserId();
-	char* user = NULL;
-	char* host = NULL;
-	int port = 9898;
-	char* pass = NULL;
-	/* raise an error if given foreign server doesn't exists */
-	ForeignServer * foreign_server = GetForeignServerByName(name, false);
-	UserMapping * user_mapping;
-	ListCell * cell;
+	Oid			userid = GetUserId();
+	char	   *user = NULL;
+	char	   *host = NULL;
+	int			port = 9898;
+	char	   *pass = NULL;
 
-	/* raise an error if the current user isn't mapped with the given foreign server */
+	/* raise an error if given foreign server doesn't exists */
+	ForeignServer *foreign_server = GetForeignServerByName(name, false);
+	UserMapping *user_mapping;
+	ListCell   *cell;
+
+	/*
+	 * raise an error if the current user isn't mapped with the given foreign
+	 * server
+	 */
 	user_mapping = GetUserMapping(userid, foreign_server->serverid);
 
 	foreach(cell, foreign_server->options)
 	{
-		DefElem * def = lfirst(cell);
+		DefElem    *def = lfirst(cell);
 
 		if (strcmp(def->defname, "host") == 0)
 		{
@@ -91,7 +98,7 @@ connect_to_server_from_foreign_server(char * name)
 
 	foreach(cell, user_mapping->options)
 	{
-		DefElem * def = lfirst(cell);
+		DefElem    *def = lfirst(cell);
 
 		if (strcmp(def->defname, "user") == 0)
 		{
@@ -103,7 +110,7 @@ connect_to_server_from_foreign_server(char * name)
 		}
 	}
 
-	return connect_to_server(host,port,user,pass);
+	return connect_to_server(host, port, user, pass);
 }
 
 /**
@@ -116,29 +123,33 @@ connect_to_server_from_foreign_server(char * name)
 Datum
 _pcp_node_info(PG_FUNCTION_ARGS)
 {
-	int16  nodeID = PG_GETARG_INT16(0);
-	char  *host_or_srv = text_to_cstring(PG_GETARG_TEXT_PP(1));
+	int16		nodeID = PG_GETARG_INT16(0);
+	char	   *host_or_srv = text_to_cstring(PG_GETARG_TEXT_PP(1));
 
-	PCPConnInfo* pcpConnInfo;
-	PCPResultInfo* pcpResInfo;
+	PCPConnInfo *pcpConnInfo;
+	PCPResultInfo *pcpResInfo;
 
-	BackendInfo * backend_info = NULL;
-	Datum values[4]; /* values to build the returned tuple from */
-	bool nulls[] = {false, false, false, false};
-	TupleDesc tupledesc;
-	HeapTuple tuple;
+	BackendInfo *backend_info = NULL;
+	Datum		values[7];		/* values to build the returned tuple from */
+	bool		nulls[] = {false, false, false, false, false, false, false};
+	TupleDesc	tupledesc;
+	HeapTuple	tuple;
+	struct tm	tm;
+	char		datebuf[20];
 
 	if (nodeID < 0 || nodeID >= MAX_NUM_BACKENDS)
 		ereport(ERROR, (0, errmsg("NodeID is out of range.")));
 
 	if (PG_NARGS() == 5)
 	{
-		char *user, *pass;
-		int port;
+		char	   *user,
+				   *pass;
+		int			port;
+
 		port = PG_GETARG_INT16(2);
 		user = text_to_cstring(PG_GETARG_TEXT_PP(3));
 		pass = text_to_cstring(PG_GETARG_TEXT_PP(4));
-		pcpConnInfo = connect_to_server(host_or_srv,port,user,pass);
+		pcpConnInfo = connect_to_server(host_or_srv, port, user, pass);
 	}
 	else if (PG_NARGS() == 2)
 	{
@@ -149,28 +160,32 @@ _pcp_node_info(PG_FUNCTION_ARGS)
 		ereport(ERROR, (0, errmsg("Wrong number of argument.")));
 	}
 
-	pcpResInfo = pcp_node_info(pcpConnInfo,nodeID);
+	pcpResInfo = pcp_node_info(pcpConnInfo, nodeID);
 	if (pcpResInfo == NULL || PCPResultStatus(pcpResInfo) != PCP_RES_COMMAND_OK)
 	{
-		char *error = pcp_get_last_error(pcpConnInfo)? pstrdup(pcp_get_last_error(pcpConnInfo)):NULL;
+		char	   *error = pcp_get_last_error(pcpConnInfo) ? pstrdup(pcp_get_last_error(pcpConnInfo)) : NULL;
+
 		pcp_disconnect(pcpConnInfo);
 		pcp_free_connection(pcpConnInfo);
-		ereport(ERROR,(0,
-					errmsg("failed to get node information"),
-					   errdetail("%s\n",error?error:"unknown reason")));
+		ereport(ERROR, (0,
+						errmsg("failed to get node information"),
+						errdetail("%s\n", error ? error : "unknown reason")));
 	}
 
 	/**
 	 * Construct a tuple descriptor for the result rows.
 	 **/
-	tupledesc = CreateTemplateTupleDesc(4, false);
+	tupledesc = CreateTemplateTupleDesc(7, false);
 	TupleDescInitEntry(tupledesc, (AttrNumber) 1, "hostname", TEXTOID, -1, 0);
 	TupleDescInitEntry(tupledesc, (AttrNumber) 2, "port", INT4OID, -1, 0);
 	TupleDescInitEntry(tupledesc, (AttrNumber) 3, "status", TEXTOID, -1, 0);
 	TupleDescInitEntry(tupledesc, (AttrNumber) 4, "weight", FLOAT4OID, -1, 0);
+	TupleDescInitEntry(tupledesc, (AttrNumber) 5, "role", TEXTOID, -1, 0);
+	TupleDescInitEntry(tupledesc, (AttrNumber) 6, "replication_delay", INT8OID, -1, 0);
+	TupleDescInitEntry(tupledesc, (AttrNumber) 7, "last_status_change", TIMESTAMPOID, -1, 0);
 	tupledesc = BlessTupleDesc(tupledesc);
 
-	backend_info = (BackendInfo *) pcp_get_binary_data(pcpResInfo,0);
+	backend_info = (BackendInfo *) pcp_get_binary_data(pcpResInfo, 0);
 
 	/* set values */
 	values[0] = CStringGetTextDatum(backend_info->backend_hostname);
@@ -193,9 +208,23 @@ _pcp_node_info(PG_FUNCTION_ARGS)
 			break;
 	}
 	nulls[2] = false;
-	values[3] = Float8GetDatum(backend_info->backend_weight/RAND_MAX);
+	values[3] = Float8GetDatum(backend_info->backend_weight / RAND_MAX);
 	nulls[3] = false;
 
+	nulls[4] = false;
+	values[4] = backend_info->role == ROLE_PRIMARY ? CStringGetTextDatum("Primary") : CStringGetTextDatum("Standby");
+
+	nulls[5] = false;
+	values[5] = Int64GetDatum(backend_info->standby_delay);
+
+	nulls[6] = false;
+	localtime_r(&backend_info->status_changed_time, &tm);
+	strftime(datebuf, sizeof(datebuf), "%F %T", &tm);
+	values[6] = DatumGetTimestamp(DirectFunctionCall3(timestamp_in,
+													  CStringGetDatum(datebuf),
+													  ObjectIdGetDatum(InvalidOid),
+													  Int32GetDatum(-1)));
+
 	pcp_disconnect(pcpConnInfo);
 	pcp_free_connection(pcpConnInfo);
 
@@ -218,18 +247,18 @@ _pcp_pool_status(PG_FUNCTION_ARGS)
 {
 	MemoryContext oldcontext;
 	FuncCallContext *funcctx;
-	int32 nrows;
-	int32 call_cntr;
-	int32 max_calls;
+	int32		nrows;
+	int32		call_cntr;
+	int32		max_calls;
 	AttInMetadata *attinmeta;
-	PCPConnInfo* pcpConnInfo;
-	PCPResultInfo* pcpResInfo;
+	PCPConnInfo *pcpConnInfo;
+	PCPResultInfo *pcpResInfo;
 
 	/* stuff done only on the first call of the function */
 	if (SRF_IS_FIRSTCALL())
 	{
-		TupleDesc tupdesc;
-		char * host_or_srv = text_to_cstring(PG_GETARG_TEXT_PP(0));
+		TupleDesc	tupdesc;
+		char	   *host_or_srv = text_to_cstring(PG_GETARG_TEXT_PP(0));
 
 		/* create a function context for cross-call persistence */
 		funcctx = SRF_FIRSTCALL_INIT();
@@ -239,12 +268,14 @@ _pcp_pool_status(PG_FUNCTION_ARGS)
 
 		if (PG_NARGS() == 4)
 		{
-			char *user, *pass;
-			int port;
+			char	   *user,
+					   *pass;
+			int			port;
+
 			port = PG_GETARG_INT16(1);
 			user = text_to_cstring(PG_GETARG_TEXT_PP(2));
 			pass = text_to_cstring(PG_GETARG_TEXT_PP(3));
-			pcpConnInfo = connect_to_server(host_or_srv,port,user,pass);
+			pcpConnInfo = connect_to_server(host_or_srv, port, user, pass);
 
 		}
 		else if (PG_NARGS() == 1)
@@ -260,14 +291,15 @@ _pcp_pool_status(PG_FUNCTION_ARGS)
 		pcpResInfo = pcp_pool_status(pcpConnInfo);
 		if (pcpResInfo == NULL || PCPResultStatus(pcpResInfo) != PCP_RES_COMMAND_OK)
 		{
-			char *error = pcp_get_last_error(pcpConnInfo)? pstrdup(pcp_get_last_error(pcpConnInfo)):NULL;
+			char	   *error = pcp_get_last_error(pcpConnInfo) ? pstrdup(pcp_get_last_error(pcpConnInfo)) : NULL;
+
 			pcp_disconnect(pcpConnInfo);
 			pcp_free_connection(pcpConnInfo);
 
 			MemoryContextSwitchTo(oldcontext);
-			ereport(ERROR,(0,
-						   errmsg("failed to get pool status"),
-						   errdetail("%s\n",error?error:"unknown reason")));
+			ereport(ERROR, (0,
+							errmsg("failed to get pool status"),
+							errdetail("%s\n", error ? error : "unknown reason")));
 		}
 
 		nrows = pcp_result_slot_count(pcpResInfo);
@@ -309,16 +341,16 @@ _pcp_pool_status(PG_FUNCTION_ARGS)
 	call_cntr = funcctx->call_cntr;
 	max_calls = funcctx->max_calls;
 
-	pcpConnInfo = (PCPConnInfo*) funcctx->user_fctx;
-	pcpResInfo = (PCPResultInfo*) pcpConnInfo->pcpResInfo;
+	pcpConnInfo = (PCPConnInfo *) funcctx->user_fctx;
+	pcpResInfo = (PCPResultInfo *) pcpConnInfo->pcpResInfo;
 	attinmeta = funcctx->attinmeta;
 
 	if (call_cntr < max_calls)	/* executed while there is more left to send */
 	{
-		char * values[3];
-		HeapTuple tuple;
-		Datum result;
-		POOL_REPORT_CONFIG *status = (POOL_REPORT_CONFIG *)pcp_get_binary_data(pcpResInfo, call_cntr);
+		char	   *values[3];
+		HeapTuple	tuple;
+		Datum		result;
+		POOL_REPORT_CONFIG *status = (POOL_REPORT_CONFIG *) pcp_get_binary_data(pcpResInfo, call_cntr);
 
 		values[0] = pstrdup(status->name);
 		values[1] = pstrdup(status->value);
@@ -350,20 +382,22 @@ _pcp_pool_status(PG_FUNCTION_ARGS)
 Datum
 _pcp_node_count(PG_FUNCTION_ARGS)
 {
-	char * host_or_srv = text_to_cstring(PG_GETARG_TEXT_PP(0));
-	int16 node_count = 0;
+	char	   *host_or_srv = text_to_cstring(PG_GETARG_TEXT_PP(0));
+	int16		node_count = 0;
 
-	PCPConnInfo* pcpConnInfo;
-	PCPResultInfo* pcpResInfo;
+	PCPConnInfo *pcpConnInfo;
+	PCPResultInfo *pcpResInfo;
 
 	if (PG_NARGS() == 4)
 	{
-		char *user, *pass;
-		int port;
+		char	   *user,
+				   *pass;
+		int			port;
+
 		port = PG_GETARG_INT16(1);
 		user = text_to_cstring(PG_GETARG_TEXT_PP(2));
 		pass = text_to_cstring(PG_GETARG_TEXT_PP(3));
-		pcpConnInfo = connect_to_server(host_or_srv,port,user,pass);
+		pcpConnInfo = connect_to_server(host_or_srv, port, user, pass);
 	}
 	else if (PG_NARGS() == 1)
 	{
@@ -378,12 +412,13 @@ _pcp_node_count(PG_FUNCTION_ARGS)
 
 	if (pcpResInfo == NULL || PCPResultStatus(pcpResInfo) != PCP_RES_COMMAND_OK)
 	{
-		char *error = pcp_get_last_error(pcpConnInfo)? pstrdup(pcp_get_last_error(pcpConnInfo)):NULL;
+		char	   *error = pcp_get_last_error(pcpConnInfo) ? pstrdup(pcp_get_last_error(pcpConnInfo)) : NULL;
+
 		pcp_disconnect(pcpConnInfo);
 		pcp_free_connection(pcpConnInfo);
-		ereport(ERROR,(0,
-					   errmsg("failed to get node count"),
-					   errdetail("%s\n",error?error:"unknown reason")));
+		ereport(ERROR, (0,
+						errmsg("failed to get node count"),
+						errdetail("%s\n", error ? error : "unknown reason")));
 	}
 
 	node_count = pcp_get_int_data(pcpResInfo, 0);
@@ -404,23 +439,25 @@ _pcp_node_count(PG_FUNCTION_ARGS)
 Datum
 _pcp_attach_node(PG_FUNCTION_ARGS)
 {
-	int16  nodeID = PG_GETARG_INT16(0);
-	char * host_or_srv = text_to_cstring(PG_GETARG_TEXT_PP(1));
+	int16		nodeID = PG_GETARG_INT16(0);
+	char	   *host_or_srv = text_to_cstring(PG_GETARG_TEXT_PP(1));
 
-	PCPConnInfo* pcpConnInfo;
-	PCPResultInfo* pcpResInfo;
+	PCPConnInfo *pcpConnInfo;
+	PCPResultInfo *pcpResInfo;
 
 	if (nodeID < 0 || nodeID >= MAX_NUM_BACKENDS)
 		ereport(ERROR, (0, errmsg("NodeID is out of range.")));
 
 	if (PG_NARGS() == 5)
 	{
-		char *user, *pass;
-		int port;
+		char	   *user,
+				   *pass;
+		int			port;
+
 		port = PG_GETARG_INT16(2);
 		user = text_to_cstring(PG_GETARG_TEXT_PP(3));
 		pass = text_to_cstring(PG_GETARG_TEXT_PP(4));
-		pcpConnInfo = connect_to_server(host_or_srv,port,user,pass);
+		pcpConnInfo = connect_to_server(host_or_srv, port, user, pass);
 	}
 	else if (PG_NARGS() == 2)
 	{
@@ -431,16 +468,17 @@ _pcp_attach_node(PG_FUNCTION_ARGS)
 		ereport(ERROR, (0, errmsg("Wrong number of argument.")));
 	}
 
-	pcpResInfo = pcp_attach_node(pcpConnInfo,nodeID);
+	pcpResInfo = pcp_attach_node(pcpConnInfo, nodeID);
 
 	if (pcpResInfo == NULL || PCPResultStatus(pcpResInfo) != PCP_RES_COMMAND_OK)
 	{
-		char *error = pcp_get_last_error(pcpConnInfo)? pstrdup(pcp_get_last_error(pcpConnInfo)):NULL;
+		char	   *error = pcp_get_last_error(pcpConnInfo) ? pstrdup(pcp_get_last_error(pcpConnInfo)) : NULL;
+
 		pcp_disconnect(pcpConnInfo);
 		pcp_free_connection(pcpConnInfo);
-		ereport(ERROR,(0,
-					   errmsg("failed to attach node"),
-					   errdetail("%s\n",error?error:"unknown reason")));
+		ereport(ERROR, (0,
+						errmsg("failed to attach node"),
+						errdetail("%s\n", error ? error : "unknown reason")));
 	}
 
 	pcp_disconnect(pcpConnInfo);
@@ -461,24 +499,26 @@ _pcp_attach_node(PG_FUNCTION_ARGS)
 Datum
 _pcp_detach_node(PG_FUNCTION_ARGS)
 {
-	int16 nodeID = PG_GETARG_INT16(0);
-	bool gracefully = PG_GETARG_BOOL(1);
-	char * host_or_srv = text_to_cstring(PG_GETARG_TEXT_PP(2));
+	int16		nodeID = PG_GETARG_INT16(0);
+	bool		gracefully = PG_GETARG_BOOL(1);
+	char	   *host_or_srv = text_to_cstring(PG_GETARG_TEXT_PP(2));
 
-	PCPConnInfo* pcpConnInfo;
-	PCPResultInfo* pcpResInfo;
+	PCPConnInfo *pcpConnInfo;
+	PCPResultInfo *pcpResInfo;
 
 	if (nodeID < 0 || nodeID >= MAX_NUM_BACKENDS)
 		ereport(ERROR, (0, errmsg("NodeID is out of range.")));
 
 	if (PG_NARGS() == 6)
 	{
-		char *user, *pass;
-		int port;
+		char	   *user,
+				   *pass;
+		int			port;
+
 		port = PG_GETARG_INT16(3);
 		user = text_to_cstring(PG_GETARG_TEXT_PP(4));
 		pass = text_to_cstring(PG_GETARG_TEXT_PP(5));
-		pcpConnInfo = connect_to_server(host_or_srv,port,user,pass);
+		pcpConnInfo = connect_to_server(host_or_srv, port, user, pass);
 	}
 	else if (PG_NARGS() == 3)
 	{
@@ -491,21 +531,22 @@ _pcp_detach_node(PG_FUNCTION_ARGS)
 
 	if (gracefully)
 	{
-		pcpResInfo = pcp_detach_node_gracefully(pcpConnInfo,nodeID);
+		pcpResInfo = pcp_detach_node_gracefully(pcpConnInfo, nodeID);
 	}
 	else
 	{
-		pcpResInfo = pcp_detach_node(pcpConnInfo,nodeID);
+		pcpResInfo = pcp_detach_node(pcpConnInfo, nodeID);
 	}
 
 	if (pcpResInfo == NULL || PCPResultStatus(pcpResInfo) != PCP_RES_COMMAND_OK)
 	{
-		char *error = pcp_get_last_error(pcpConnInfo)? pstrdup(pcp_get_last_error(pcpConnInfo)):NULL;
+		char	   *error = pcp_get_last_error(pcpConnInfo) ? pstrdup(pcp_get_last_error(pcpConnInfo)) : NULL;
+
 		pcp_disconnect(pcpConnInfo);
 		pcp_free_connection(pcpConnInfo);
-		ereport(ERROR,(0,
-					   errmsg("failed to detach node"),
-					   errdetail("%s\n",error?error:"unknown reason")));
+		ereport(ERROR, (0,
+						errmsg("failed to detach node"),
+						errdetail("%s\n", error ? error : "unknown reason")));
 	}
 
 	pcp_disconnect(pcpConnInfo);
