@@ -8,7 +8,7 @@ PSQL=$PGBIN/psql
 
 function getnode()
 {
-	grep $1 log/pgpool.log | grep SELECT | awk '{print $9}'
+	grep $1 log/pgpool.log | grep SELECT | grep LOG: |awk '{print $9}'
 }
 
 rm -fr $TESTDIR
@@ -51,6 +51,7 @@ test `getnode "case3"` -eq 0 || ok=ng
 echo "allow_sql_comments = on" >> etc/pgpool.conf
 
 ./pgpool_reload
+sleep 1
 
 wait_for_pgpool_startup
 
